@@ -4,7 +4,7 @@ Equilibrium utilities library
 
 import sys, time, logging, datetime
 import numpy as np
-from scipy.ndimage.interpolation import map_coordinates
+from scipy.ndimage import map_coordinates
 from scipy.interpolate import UnivariateSpline, interp1d, InterpolatedUnivariateSpline, RectBivariateSpline, LinearNDInterpolator
 
 logger = logging.getLogger('aug_sfutils.mapeq')
