@@ -7,7 +7,7 @@ import ctypes as ct
 import pandas as pd
 
 #  SFfmt, SFtyp,       np,         ct, struc,   descr
-typesList = [ \
+dtypesList = [ \
   [  1,   0,    np.int8,   ct.c_byte, 'b', 'BYTE'        ],
   [  2,   6,        str,   ct.c_char, 'c', 'CHAR'        ],
   [  3,  11,   np.int16,  ct.c_int16, 'h', 'SHORT_INT'   ],
@@ -25,7 +25,32 @@ typesList = [ \
   [ 15,  14,  np.uint64, ct.c_uint64, 'Q', 'U_LONGLONG'  ],
 ]
 
-dataTypes = pd.DataFrame(typesList, columns = ['SFfmt', 'SFtyp', 'np', 'ct', 'struc', 'descr'])
+dtf = pd.DataFrame(dtypesList, columns = ['SFfmt', 'SFtyp', 'np', 'ct', 'struc', 'descr'])
+
+
+SFOtypeList = [\
+    [ 1, 'Diagnostic' , '>4s2H2I4H' , ('diag_code', 'num_objs', 'diag_type', 'shot_nr', 'c_time', 'up_limit', 'exp', 'version', 's_type')],
+    [ 2, 'List'       , '>4H16x'    , ('dataFormat', 'nitems', 'ordering', 'list_type')],
+    [ 3, 'Device'     , '>4H2I2HI'  , ('dataFormat', 'acquseq', 'nitems', 'dev_type', 'dev_addr', 'n_chan', 'task', 'dev_num', 'n_steps')],
+    [ 4, 'ParamSet'   , '>4x2H16x'  , ('nitems', 'cal_type')],
+    [ 5, 'MapFunc'    , '>4x2H8xH6x', ('nitems', 'map_type', 'task')],
+    [ 6, 'SignalGroup', '>3Hh4I'    , ('dataFormat', 'physunit', 'num_dims', 'stat_ext', 'index1', 'index2', 'index3', 'index4')],
+    [ 7, 'Signal'     , '>3Hh4I'    , ('dataFormat', 'physunit', 'num_dims', 'stat_ext', 'index1', 'index2', 'index3', 'index4')],
+    [ 8, 'TimeBase'   , '>4HI4x2I'  , ('dataFormat', 'burstcount', 'event', 'tbase_type', 's_rate', 'n_pre', 'n_steps')],
+    [ 9, 'SFList'     , '>2xH20x'   , ('nitems', )],
+    [10, 'Algorithm'  , '>8x8s8s'   , ('hostname', 'date')],
+    [11, 'UpdateSet'  , '>2xHi8x2I' , ('nitems', 'input_vals', 'next_index', 'size')],
+    [12, 'LocTimer'   , '>2H16xI'   , ('dataFormat', 'resolution', 'size')],
+    [13, 'AreaBase'   , '>4H4I'     , ('dataFormat', 'physunit1', 'physunit2', 'physunit3', 'size_x', 'size_y', 'size_z', 'n_steps')],
+    [14, 'Qualifier'  , '>H2x2H4I'  , ('dataFormat', 'num_dims', 'qsub_typ', 'index_4', 'index_3', 'index_2', 'max_sections')],
+    [15, 'ModObj'     , '>H22x'     , ('nitems', )],
+    [16, 'MapExtd'    , '>2H2I2H2I' , ('nitems', 'mapalg', 'tbeg', 'tend', 'val_0', 'val_1', 'val_2', 'val_3')],
+    [17, 'Resource'   , '>2H20x'    , ('num_cpus', 'first_cpu')],
+    [18, 'ADDRLEN'    , '>H22x'     , ('addrlen', )] ]
+
+sfof = pd.DataFrame(SFOtypeList, columns = ['SFOtypeID', 'SFOtypeLabel', 'SFOtypeStruc', 'SFOattrs'])
+
+header_sfmt = '>8s3Hh8H2I24s64s'
 
 # CHAR length for string variables
 fmt2len = {2: 1, 1794: 8, 3842: 16, 7938: 32, 12034: 48, 16130: 64, 18178: 72}
@@ -58,42 +83,6 @@ unit_d = { \
   129:'J/m^3', 130:'1/cm^3', 131:'Nm/m^3', 132:'Web', 133:'Web/rad', 134:'Tm' \
 }
 
-def typeMap(col_in, col_out, val_in):
-    return dataTypes.loc[dataTypes[col_in] == val_in][col_out].values[0]
-
-
-def typeLength(SFfmt):
-
-    if SSFfmt in fmt2len.keys(): # char variable
-        type_len = fmt2len[dfmt]
-    else: # numerical variable
-        struc = typeMap('SFfmt', 'struc', SFfmt)
-        type_len = np.dtype(struc).itemsize
-
-    return type_len
-
-
-class ObjectID:
-
-    Diagnostic  = 1
-    List        = 2
-    Device      = 3
-    ParamSet    = 4
-    MapFunc     = 5
-    SignalGroup = 6
-    Signal      = 7
-    TimeBase    = 8
-    SFList      = 9
-    Algorithm   = 10
-    UpdateSet   = 11
-    LocTimer    = 12
-    AreaBase    = 13
-    Qualifier   = 14
-    ModObj      = 15
-    MapExtd     = 16
-    Resource    = 17
-    ADDRLEN     = 18
-
 
 class CalibType:
 
@@ -112,7 +101,52 @@ class TimebaseType:
     Condensed  = 5
 
 
-if __name__ == '__main__':
+def typeMap(col_in, col_out, val_in):
+    return dtf.loc[dtf[col_in] == val_in][col_out].values[0]
 
-    print(typeMap('SFfmt', 'np', 1))
-    print(typeMap('descr', 'SFtyp', 'U_SHORT'))
+
+def SFOmap(col_in, col_out, val_in):
+    return sfof.loc[sfof[col_in] == val_in][col_out].values[0]
+
+
+olbl   = pd.Series(sfof.SFOtypeLabel.values, index=sfof.SFOtypeID   ).to_dict()
+ostruc = pd.Series(sfof.SFOtypeStruc.values, index=sfof.SFOtypeLabel).to_dict()
+oid    = pd.Series(sfof.SFOtypeID.values   , index=sfof.SFOtypeLabel).to_dict()
+oattr  = pd.Series(sfof.SFOattrs.values    , index=sfof.SFOtypeLabel).to_dict()
+
+
+def typeLength(SFfmt):
+    if SFfmt in fmt2len.keys(): # char variable
+        type_len = fmt2len[SFfmt]
+    else: # numerical variable
+        struc = typeMap('SFfmt', 'struc', SFfmt)
+        type_len = np.dtype(struc).itemsize
+    return type_len
+
+calibLabel = {val: key for key, val in vars(CalibType).items()}
+#timebaseLabel = {val: key for key, val in var(TimebaseType).items() if '__' not in key}
+timebaseLabel = {val: key for key, val in vars(TimebaseType).items()}
+
+
+def arrayShape(sfo):
+
+    SFOlbl = sfo.SFOtypeLabel
+
+    if SFOlbl in ('SignalGroup', 'Signal'):
+        shape_arr = np.array(sfo.index[::-1][:sfo.num_dims])
+    elif SFOlbl == 'TimeBase':
+        shape_arr = np.array([sfo.n_steps])
+    elif SFOlbl == 'AreaBase':
+        shape_arr = np.array([sfo.size_x + sfo.size_y + sfo.size_z, sfo.n_steps])
+    else:
+        return None
+
+    return shape_arr
+
+
+def objectLength(sfo):
+
+    shape_arr = arrayShape(sfo)
+    type_len  = typeLength(sfo.dataFormat)
+
+    return np.prod(shape_arr) * type_len
