@@ -2,7 +2,7 @@ import os, datetime, logging
 import numpy as np
 
 from aug_sfutils import sfmap, sfobj, manage_ed, parse_kwargs, str_byt, libddc, getlastshot
-from aug_sfutils.sf_read import SF_READ
+from aug_sfutils.shotfile import SHOTFILE as SF
 
 
 logger = logging.getLogger('aug_sfutils.sfread')
@@ -123,7 +123,7 @@ class SFREAD:
             return
 
         logger.info('Fetching SF %s', self.sfpath)
-        self.sf = SF_READ(self.sfpath)
+        self.sf = SF(self.sfpath)
         self.status = (self.sf is not None)
 
         self.cache = {}
